@@ -70,7 +70,8 @@ PARAMETER_MAPPING = {
     'upgrade_cleanup': 'UpgradeRemoveUnusedPackages',
     'local_subnet': 'UndercloudCtlplaneLocalSubnet',
     'enable_routed_networks': 'UndercloudEnableRoutedNetworks',
-    'local_interface': 'NeutronPublicInterface'
+    'local_interface': 'NeutronPublicInterface',
+    'docker_bip': 'DockerNetworkOptions'
 }
 
 SUBNET_PARAMETER_MAPPING = {
@@ -168,18 +169,21 @@ def _process_drivers_and_hardware_types(conf, env):
         if hw_type in enabled_hardware_types:
             vendor_interfaces.add(iface)
 
+    power_interfaces = mgmt_interfaces.copy()
+    # The snmp hardware type uses noop management and snmp power; noop
+    # management is also used by ipmi and staging hardware types.
+    mgmt_interfaces.add('noop')
+    if 'snmp' in enabled_hardware_types:
+        power_interfaces.add('snmp')
+
     env['IronicEnabledHardwareTypes'] = sorted(enabled_hardware_types)
 
     env['IronicEnabledBootInterfaces'] = sorted(boot_interfaces)
     env['IronicEnabledManagementInterfaces'] = sorted(mgmt_interfaces)
+    env['IronicEnabledPowerInterfaces'] = sorted(power_interfaces)
     env['IronicEnabledRaidInterfaces'] = sorted(raid_interfaces)
     env['IronicEnabledVendorInterfaces'] = sorted(vendor_interfaces)
 
-    # The snmp hardware type uses fake management and snmp power
-    if 'snmp' in enabled_hardware_types:
-        mgmt_interfaces.add('snmp')
-    env['IronicEnabledPowerInterfaces'] = sorted(mgmt_interfaces)
-
 
 def _process_ipa_args(conf, env):
     """Populate the environment with IPA kernal args ."""
@@ -302,10 +306,12 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=False,
         env_data['SELinuxMode'] = 'permissive'
 
     if CONF.get('undercloud_ntp_servers', None):
-        env_data['NtpServer'] = CONF['undercloud_ntp_servers'][0]
+        env_data['NtpServer'] = CONF['undercloud_ntp_servers']
 
-    if CONF.get('enable_validations', False) and not no_validations:
-        env_data['EnableValidations'] = CONF['enable_validations']
+    if CONF.get('enable_validations', False):
+        env_data['UndercloudConfigFilePath'] = constants.UNDERCLOUD_CONF_PATH
+        if not no_validations:
+            env_data['EnableValidations'] = CONF['enable_validations']
 
     if CONF.get('overcloud_domain_name', None):
         env_data['NeutronDnsDomain'] = CONF['overcloud_domain_name']
@@ -325,8 +331,13 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=False,
     env_data['LocalContainerRegistry'] = CONF['local_ip'].split('/')[0]
 
     if CONF['additional_architectures']:
+        # In queens (instack-undercloud) we used this to setup additional
+        # architectures.  For rocky+ we want to pass a list and be smarter in
+        # THT.  We can remove this in 'T' when we get there.
         for arch in CONF['additional_architectures']:
             env_data['EnableArchitecture%s' % arch.upper()] = True
+        env_data['AdditionalArchitectures'] = \
+            ','.join(CONF['additional_architectures'])
 
     if CONF.get('local_ip', None):
         deploy_args.append('--local-ip=%s' % CONF['local_ip'])
@@ -361,7 +372,9 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=False,
     # These should be loaded first so we can override all the bits later
     deploy_args += [
         "-e", os.path.join(tht_templates, "environments/docker.yaml"),
-        "-e", os.path.join(tht_templates, "environments/undercloud.yaml")]
+        "-e", os.path.join(tht_templates, "environments/undercloud.yaml"),
+        "-e", os.path.join(tht_templates, "environments/use-dns-for-vips.yaml")
+        ]
 
     # If a container images file is used, copy it into the tempdir to make it
     # later into other deployment artifacts and user-provided files.
@@ -395,7 +408,7 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=False,
 
     if CONF.get('enable_zaqar'):
         deploy_args += ['-e', os.path.join(
-            tht_templates, "environments/services/zaqar.yaml")]
+            tht_templates, "environments/services/zaqar-swift-backend.yaml")]
 
     if CONF.get('enable_telemetry'):
         for env_file in TELEMETRY_DOCKER_ENV_YAML:
@@ -465,9 +478,6 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=False,
 
         deploy_args += [
             '-e', endpoint_environment,
-            '-e', os.path.join(
-                tht_templates,
-                'environments/use-dns-for-vips.yaml'),
             '-e', os.path.join(
                 tht_templates,
                 'environments/services/undercloud-haproxy.yaml'),
@@ -579,8 +589,9 @@ def prepare_undercloud_deploy(upgrade=False, no_validations=False,
     if force_stack_update:
         deploy_args += ["--force-stack-update"]
 
-    cmd = ["sudo", "openstack", "tripleo", "deploy", "--standalone",
-           "--standalone-role", "Undercloud", "--stack", "undercloud"]
+    cmd = ["sudo", "--preserve-env", "openstack", "tripleo", "deploy",
+           "--standalone", "--standalone-role", "Undercloud", "--stack",
+           "undercloud"]
     cmd += deploy_args[:]
 
     # In dry-run, also report the expected heat stack virtual state/action
